#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Author: Pocholo
# Date: 3/20/21

# Inspired by a cONBAGUI'S tutorial found here: https://www.deviantart.com/conbagui/art/Bubble-Wrap-Text-447829913

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.   

from gimpfu import *
import gettext
import os,sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_create_a_bubble_wrap_text(pattern, customtext, fontname, size):
          
    #Initiates the temporary state
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
	     
    #Create the background
    img = pdb.gimp_image_new(1600, 1200, RGB_IMAGE)   
    boxLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Wood box", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, boxLayer, 0)
    pdb.gimp_layer_add_alpha(boxLayer)
    pdb.gimp_context_set_pattern(pattern)
    pdb.gimp_drawable_fill(boxLayer, FILL_PATTERN)
    
    pdb.gimp_display_new(img)
    
    #Create the cork texture
    corklayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Cork", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, corklayer, None, 0)
    pdb.gimp_layer_add_alpha(corklayer)
    pdb.gimp_context_set_foreground((216, 148, 100))
    pdb.gimp_drawable_fill(corklayer, FILL_FOREGROUND)
    pdb.plug_in_hsv_noise(img, corklayer, 1, 0, 50, 255)
    pdb.plug_in_gmic_qt(img, corklayer, 1, 0, "-v - -fx_crystal 50,2,40")
    
    #Create the box
    pdb.gimp_layer_scale(boxLayer, 1440, 1050, TRUE)
    pdb.gimp_layer_resize_to_image_size(boxLayer)
    pdb.gimp_image_select_item(img, 2, boxLayer)
    pdb.gimp_drawable_edit_clear(corklayer)
    pdb.gimp_selection_none(img)
                                            
    #Text outer glow
    pdb.python_layerfx_outer_glow(img, corklayer, 
								       (0, 0, 0),	#color 
										  	 80,    #opacity
											   0,   #contou - linear
											   0,   #noise
											   3,   #mode - multiply
											  60,   #spread 
											  35,  #size 
											TRUE,   #knockout 
										   FALSE)   #merge
                                           
    outerglowLayer = pdb.gimp_image_get_layer_by_name(img, "Cork-outerglow")
    pdb.gimp_image_set_active_layer(img, outerglowLayer)
    pdb.gimp_layer_resize_to_image_size(outerglowLayer)
    pdb.plug_in_gauss(img, outerglowLayer, 35, 35, 1)
    boxLayer = pdb.gimp_image_merge_visible_layers(img, 2)
        
    #Create the bubble wrap pattern
    bubbleLayer = pdb.gimp_layer_new(img, 395, 345, RGBA_IMAGE, "Bubbles wrap", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, bubbleLayer, 0)
    pdb.gimp_layer_add_alpha(bubbleLayer)
    pdb.gimp_drawable_fill(bubbleLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Bubble wrap 1")
    pdb.gimp_context_set_brush_size(200)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.1)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_paintbrush_default(bubbleLayer, 2, [197, 170])
    pdb.gimp_brushes_set_brush("Bubble wrap 2")       
    pdb.gimp_paintbrush_default(bubbleLayer, 2, [0, 170])
    pdb.gimp_paintbrush_default(bubbleLayer, 2, [395, 170])
    pdb.gimp_brushes_set_brush("Bubble wrap 3")                                 
    pdb.gimp_paintbrush_default(bubbleLayer, 2, [100, 345])
    pdb.gimp_paintbrush_default(bubbleLayer, 2, [100, 0])
    pdb.gimp_brushes_set_brush("Bubble wrap 4")                                 
    pdb.gimp_paintbrush_default(bubbleLayer, 2, [295, 0])
    pdb.gimp_paintbrush_default(bubbleLayer, 2, [295, 345])
    pdb.gimp_layer_scale(bubbleLayer, 80, 70, TRUE)
        
    #Copy the pattern and delete the layer
    non_empty = pdb.gimp_edit_copy(bubbleLayer)
    pdb.gimp_image_remove_layer(img, bubbleLayer)
    
    #Create the text
    boxLayer = pdb.gimp_image_get_layer_by_name(img, "Wood box")
    textW = size/0.3
    textH = size/0.4
    pdb.gimp_context_set_foreground("White")
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 10, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 10)
    pdb.gimp_text_layer_set_line_spacing(textLayer, 0)
        
    #Resize text
    pdb.gimp_layer_scale(textLayer, textW, textH, TRUE)
                
    #Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/2
    centery=h/2
    adw=wh/2
    adh=hh/2
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    #Rename the text layer and reduce opacity
    pdb.gimp_item_set_name(textLayer, "Text")
    pdb.gimp_layer_set_opacity(textLayer, 0)
    
    #Drop Shadow
    pdb.python_layerfx_drop_shadow(img, textLayer,
								        ("Black"),   #color
											   30,   #opacity
												0,   #contour - linear
												0,   #noise
												3,   #mode - multiply   
												0,   #spread
											    4,   #size
											   90,   #offset_angle
												7,   #offset_distance
											 TRUE,   #knockout
											FALSE)   #merge
    
    #Text inner shadow
    pdb.python_layerfx_inner_shadow(img, textLayer, 
                                         ("White"),   #color
                                                14,   #opacity
                                                 0,   #contour - linear
                                                 0,   #noise
                                                16,   #mode - dodge
                                                 1,   #source
                                                 0,   #choke 
                                                25,   #size 
                                                90,   #offset_angle 
                                                35,   #offset_distance 
                                              FALSE)  #merge
    
    #Pattern overlay		
    pdb.python_layerfx_pattern_overlay(img, textLayer, 
                                 _("Clipboard Image"),   #pattern
                                                   75,   #opacity
                                                    0,   #mode - normal
                                                  100,   #scale
                                                    0,   #interpolation
                                                FALSE)   #merge
    
    #Bevel and emboss
    pdb.python_layerfx_bevel_emboss(img, textLayer, 
                                                 1,   #style inner bevel
                                                65,   #depth 
                                                 0,   #direction 
                                                40,   #size 
                                                 5,   #soften
                                                90,   #angle 
                                                20,   #altitude 
                                                 8,   #gloss_contour - ring double
                                         ("White"),   #highlight_color 
                                                 4,   #highlight_mode screen
                                               48,   #highlight_opacity
                                    (225,225,225),   #shadow_color 
                                                3,   #shadow_mode - multyply
                                               22,   #shadow_opacity 
                                                6,   #surface_contour - half round
                                            FALSE,   #use_texture 
                                      "Dried mud",   #pattern 
                                              100,   #scale 
                                              100,   #tex_depth 
                                             TRUE,   #invert 
                                            FALSE)   #merge    
    	
    boxLayer = pdb.gimp_image_get_layer_by_name(img, "Wood box")
    pdb.gimp_image_set_active_layer(img, boxLayer)
        
    #Set gimp to default
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
   	
register(
    "pm_create_a_bubble_wrap_text",
    "Creates a Bubble wrap text",
    "Creates a Bubble wrap text effect",
    "Pocholo",
    "Pocholo",
    "2021",
    "Create a Bubble Wrap Text",
    "",
    [    
    (PF_PATTERN, "pattern", "Background pattern", "Kewazinga wood Texture.jpg"),
    (PF_TEXT, "customtext", "Text string", 'BUBBLE\nWRAP\nTEXT'),
    (PF_FONT, "font", "Font name", "Mousou Record__G"),
    (PF_SPINNER, "size", "Text size", 400, (400, 400, 1)),
    ],
    [],
    pm_create_a_bubble_wrap_text, menu="<Image>/Pocholo-scripts/Create a Bubble Wrap Text",
    domain=("gimp20", gimp.locale_directory))
main()						   			